#!/bin/sh
######################################################################################
# Copyright:  Copyright (c) IBM Corp. 1998,2007. All Rights Reserved.
# Time-stamp: <050212 ihoshko 19:45 apply.sh>
# Name:
#   apply.sh
# Purpose:
#   Script for update actuator platform with list of files from "files.lst", which are
#   located in archive "files.tar".
#   Support of custom hotfix appliance logic is done by using additional scripts:
#     'pre-run*.sh'  - will be executed at first, before agent stop
#     'run*.sh'      - will be executed after agent stop and package files update
#     'post-run*.sh' - will be executed after agent restart
#   Each of three mentioned file-sets will be executed in alphabetical order (i.e.
#   'run1.sh', 'run2.sh', ...).
#   Files without executable flag set are ignored.
#   In case of failure, rollback will be performed in back order.
# Arguments:
#   $1 - actuator home directory
# Notes:
#   If script argument was absent, then script will be terminated with small help
#   message about default actuator directory for different platforms.
# History:
#   050212 ihoshko Created.
#   050302 0.0.2 Luc WE0502055: Made changes to run under z/OS
#   050305 0.0.6 SVolokh: Subscript running implementation added.
#   050310 0.0.6 ihoshko: Correct rollback operation.
#   050602 0.0.8 semechko: Start/stop watchdog functionality added
######################################################################################
CEACLIENT=start-client
SUBMAPPERDIR=submapper
MSGDIR=bin
FMSG=appmsg.msg
FLIST=files.lst
FTAR=files.tar
TEMPDIR=/tmp/hotfixtemp$$
BACKUPDIR=/tmp/hotfixbck$$
PRE_APPLIED_RUNS=
IN_APPLIED_RUNS=
POST_APPLIED_RUNS=
PRE_FILES_PATTERN="pre-run*.sh"
IN_FILES_PATTERN="run*.sh"
POST_FILES_PATTERN="post-run*.sh"
SYSTEM=`uname -s`
if [ "$SYSTEM" = "OS/390" ] # this is also true with z/OS systems
then
   CEACLIENT=agent-start  # or else update eprise.product.actuator.bin, OS390/setup.sh
fi
test $SYSTEM = "HP-UX" && WATCHDOG="/sbin/init.d/cea.watchdog"
test $SYSTEM = "SunOS" && WATCHDOG="/etc/init.d/CEAWatchdog"

######################################################################################
# Name: die()
# Arguments:
#   $1 - exit code
#   $2 - exit message
# Purpose:
#   Function for program termination.
######################################################################################
die()
{
  exit_code=$1
  if [ "$exit_code" -ne "0" ]
  then
     echo 
     [ -n "$2" ] && echo "ERROR: $2"
     echo "Hotfix Installation FAILED!"
     echo

     #Exit without rallback operation.
     [ "$exit_code" -eq "2" ] && exit "$exit_code"

     echo "Installation rollback..."
     post_executable_rollback
     stop_agent || echo "Warning! Following \"in\" and \"file\" rallback doing with running agent!"
     in_executable_rollback
     rollback_files "$BACKUPDIR" "$CEAACTDIR"
     start_agent || echo "Warning! Following \"pre\" executable rallback doing with stopping agent!"
     pre_executable_rollback
  else
     echo "$2"
  fi

  if [ -d "$BACKUPDIR" ]
  then
     echo "Removing backup folder.."
     rm -Rf "$BACKUPDIR"
  fi

  if [ -d "$TEMPDIR" ]
  then
     echo "Removing temporary folder.."
     rm -Rf "$TEMPDIR"
  fi

  echo
  exit "$exit_code"
}

######################################################################################
# Name: single_executable_run()
# Arguments:
#   $1 - name of child subscript
# Purpose:
#   Execute child script and check exit code. If error - initiate rollback
######################################################################################
single_executable_run()
{
  exec="$1"
  if [ -f "./$exec" ]
  then
     if [ ! -x "./$exec" ]
     then
        chmod 774 "./$exec" || echo "Warning: Cannot change \"./$exec\" to executable mode!"
     fi
     echo "  Executing: \"$exec\" ..."
     "./$exec" || die 1 "Failed to execute \"$exec\" !"
  fi
}

######################################################################################
# Name: single_executable_rollback()
# Arguments:
#   $1 - name of child subscript
# Purpose:
#   Execute child script for rollback
######################################################################################
single_executable_rollback()
{
  exec="$1"
  if [ -f "./$exec" ]
  then
     if [ ! -x "./$exec" ]
     then
        chmod 774 "./$exec" || echo "Warning: Cannot change \"./$exec\" to executable mode!" 
     fi 
     echo "  Executing: \"$exec\" \"rollback\" ..."
     "./$exec" "rollback" || echo "Warning: Failed to execute \"$exec\" \"rollback\" !"
  fi
}

######################################################################################
# Name: pre_executables_run()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts to be executed before files update
#   and agent stop
######################################################################################
pre_executables_run()
{
  echo "Pre executables running..."
  cd "$CURDIR"
  for exec in `ls $PRE_FILES_PATTERN 2>/dev/null`
  do
     PRE_APPLIED_RUNS="$exec $PRE_APPLIED_RUNS"
     single_executable_run "$exec"
  done
}

######################################################################################
# Name: in_executables_run()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts to be executed after files update
#   and before agent start
######################################################################################
in_executables_run()
{
  echo "In executables running..."
  cd "$CURDIR"
  for exec in `ls $IN_FILES_PATTERN 2>/dev/null`
  do
     IN_APPLIED_RUNS="$exec $IN_APPLIED_RUNS"
     single_executable_run "$exec"
  done
}

######################################################################################
# Name: post_executables_run()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts to be executed after files update
#   and agent start
######################################################################################
post_executables_run()
{
  echo "Post executables running..."
  cd "$CURDIR"
  for exec in `ls $POST_FILES_PATTERN 2>/dev/null`
  do
     POST_APPLIED_RUNS="$exec $POST_APPLIED_RUNS"
     single_executable_run "$exec"
  done
}

######################################################################################
# Name: pre_executable_rollback()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts rollback for applied in this session
#   'pre-run*.sh' scripts
######################################################################################
pre_executable_rollback()
{
  echo "Pre executables rollback..."
  for exec in $PRE_APPLIED_RUNS
  do
     single_executable_rollback "$exec"
  done
  PRE_APPLIED_RUNS=
}

######################################################################################
# Name: in_executable_rollback()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts rollback for applied in this session
#   'run*.sh' scripts
######################################################################################
in_executable_rollback()
{
  echo "In executables rollback..."
  for exec in $IN_APPLIED_RUNS
  do
     single_executable_rollback "$exec"
  done
  IN_APPLIED_RUNS=
}

######################################################################################
# Name: post_executable_rollback()
# Arguments: 
#   (none)
# Purpose:
#   Find and call execution of child subscripts rollback for applied in this session
#   'post-run*.sh' scripts
#   NOTE: This function also stops agent in case when we have done any rollbacks
#   on this stage
######################################################################################
post_executable_rollback()
{
  echo "Post executables rollback..."
  for exec in $POST_APPLIED_RUNS
  do
     single_executable_rollback "$exec"
  done
  POST_APPLIED_RUNS=
}

######################################################################################
# Name: stop_agent()
# Arguments: 
#   (none)
# Purpose:
#   Stop actuator agent service
######################################################################################
stop_agent()
{
  exit_code=0
  echo "Stopping IBM Tivoli Compliance Insight Manager Actuator deamon..."
  agentID=`getAgentID`
  if [ -n "$agentID" ]
  then
     kill $agentID
     sleep 3
     agentID=`getAgentID`
     if [ -n "$agentID" ]
     then
        echo "Terminating IBM Tivoli Compliance Insight Manager Actuator deamon..."
        kill -9 $agentID
        sleep 3
        agentID=`getAgentID`
        if [ -n "$agentID" ]
        then
           echo "Cannot stop or terminate IBM Tivoli Compliance Insight Manager Actuator deamon!"
           exit_code=1
        else
           echo "IBM Tivoli Compliance Insight Manager Actuator deamon terminated."
        fi
     else
       echo "IBM Tivoli Compliance Insight Manager Actuator deamon stopped."
     fi
  else
     echo "IBM Tivoli Compliance Insight Manager Actuator deamon already stopped!"
  fi
  return $exit_code
}

######################################################################################
# Name: start_agent()
# Arguments: 
#   (none)
# Purpose:
#   Start actuator agent service
######################################################################################
start_agent()
{
  exit_code=0

  if [ "$SYSTEM" = "AIX" ]
  then
    return 0
  fi

  echo "Starting IBM Tivoli Compliance Insight Manager Actuator deamon..."
  agentID=`getAgentID`
  if [ ! -n "$agentID" ]
  then
     if [ -x "$CEAACTDIR/$CEACLIENT" ]
     then
        (cd "$CEAACTDIR" && "./$CEACLIENT")
        sleep 4
        agentID=`getAgentID`
        if [ -n "$agentID" ]
        then
           echo "IBM Tivoli Compliance Insight Manager Actuator deamon successfully STARTED!"
        else
           echo "IBM Tivoli Compliance Insight Manager Actuator deamon could not be started!"
           exit_code=1
        fi
     else
        echo "Cannot find IBM Tivoli Compliance Insight Manager Actuator executable client \"$CEAACTDIR/$CEACLIENT\"!"
        exit_code=1
     fi
  else
     echo "IBM Tivoli Compliance Insight Manager Actuator deamon already started!"
  fi
  return $exit_code
}

######################################################################################
# Name: stop_watchdog()
# Arguments: 
#   (none)
# Purpose:
#   Switch off automatically restart functionality.
#   z/OS specific - do nothing
#   AIX specific - create file named UPGRADING in $CEAACTDIR to notify start-client 
#                  that it is not necessary to start agent.
#   Other platforms: kill process "watchdog"
######################################################################################
stop_watchdog()
{ 
  test "$SYSTEM" = "OS/390" && return 0 

  echo "Switching off automatically restart functionality..."  
  if [ "$SYSTEM" = "AIX" ]
  then
    touch $CEAACTDIR/UPGRADING || return 1
    return 0
  fi
  
  $WATCHDOG stop
  return 0
}

######################################################################################
# Name: start_watchdog()
# Arguments: 
#   (none)
#   Switch on automatically restart functionality.
#   z/OS specific - do nothing
#   AIX specific - remove file named UPGRADING in $CEAACTDIR
#   Other platforms: start process "watchdog"
######################################################################################
start_watchdog()
{
  test "$SYSTEM" = "OS/390" && return 0 

  echo "Switching on automatically restart functionality..."  
  if [ "$SYSTEM" = "AIX" ]
  then
    rm -f $CEAACTDIR/UPGRADING || return 1
  
    echo "Starting IBM Tivoli Compliance Insight Manager Actuator deamon..."
    clientID=`ps -ef | grep "$CEACLIENT" | grep -v "grep $CEACLIENT" | awk '{print $2}'`
    if [ "$clientID" != "" ]
    then
      kill -9 $clientID #it causes restart agent
    else
      RESPAWN=`cat /etc/inittab | grep rc.cea.actuator | awk -F: '{print $2}'`
      if [ "$RESPAWN" = "respawn" ]
      then
        echo "There is no \"$CEACLIENT\" running!"
        echo "IBM Tivoli Compliance Insight Manager Actuator deamon could not be started!"
        return 1
      else
        echo "Starting agent" 
        cd $CEAACTDIR
        ./start-client
        cd - > /dev/null
        sleep 4
        clientID=`ps -ef | grep "../bin/agent" | grep -v "grep ../bin/agent" | awk '{print $2}'`
        if [ "$clientID" != "" ]
        then
          return 0
        else
          echo "There is no \"$CEACLIENT\" running!"
          echo "IBM Tivoli Compliance Insight Manager Actuator deamon could not be started by the hotfix!"          
          return 1
        fi
      fi  
    fi
    return 0
  fi
  
  $WATCHDOG start
  return 0
}


######################################################################################
# Name: single_file_rollback()
# Arguments:
#   $1 - backup dir
#   $2 - restore dir
#   $3 - filename for testore
# Purpose:
#   Function for single file rollback.
######################################################################################
single_file_rollback() {
  if [ -d "$2" ]
  then
     if [ -f "$1/$3.old" ]
     then 
        echo "  Rollback \"$2/$3\"..." 
        cp -f "$1/$3.old" "$2/$3" >/dev/null 2>&1 || echo "Warning: Cannot coppy to \"$2/$3\" for rollback!"
     else
        if [ -f "$1/$3.new" ]
        then 
           echo "  Removing \"$2/$3\"..." 
           rm -f "$2/$3" >/dev/null 2>&1 || echo "Warning: Cannot remove \"$2/$3\" for rollback!"
        fi
     fi
  fi
}

###################################################################################### 
# Name: rollback_files() 
# Arguments: 
#   $1 - backup dir 
#   $2 - restore dir 
# Purpose:
#   Function for files rollback. 
######################################################################################
rollback_files() {
  echo "Updated files rollback..."
  if [ -d "$1" ]
  then
     if [ ! -d "$2" ]
     then
        echo "Restore dirrectory \"$2\" does not exist!"
     else
        if [ ! -f "$CURDIR/$FLIST" ]
        then
           echo "File \"$CURDIR/$FLIST\" does not exist!"
        else
           for filewithpath in `cat "$CURDIR/$FLIST" | grep -v "^#" | sort -u`
           do
              filename=`basename "$filewithpath"`
              pckgdir=`getpath "$filewithpath"` 
              single_file_rollback "$1/$pckgdir" \
                                   "$2/$pckgdir" \
                                   "$filename"
              #Rollback message files for each executable files.
              fileBinExeStatus=`getBinExeStatus "$2/$pckgdir/$filename"` 
              if [ "$fileBinExeStatus" -eq "1" ]
              then 
                 single_file_rollback "$1/$pckgdir" \
                                      "$2/$pckgdir" \
                                      "$filename.msg"
              fi
           done
        fi
     fi
  fi
}

######################################################################################
# Name: update_file() 
# Arguments:
#   $1 - source dir 
#   $2 - target dir 
#   $3 - backup dir 
#   $4 - source filename 
#   $5 - target filename 
# Purpose: 
#   Function for single file update. 
######################################################################################
update_file() { 
  if [ ! -r "$1/$4" ]
  then 
     die 1 "There is no hotfix file \"$4\" for update \"$2/$5\"!" 
  fi
  newdelivery=0
  if [ -f "$2/$5" ]
  then
     echo "  Updating \"$2/$5\"..."
     cp "$2/$5" "$3/$5.old" >/dev/null 2>&1 || die 1 "Cannot create backup copy of \"$2/$5\" as \"$3/$5.old\"!"
     cp "$2/$5" "$2/$5.bck" >/dev/null 2>&1
     if [ $? -ne 0 ]
     then
        rm -f "$3/$5.old"
        die 1 "Cannot create backup copy of \"$2/$5\" as \"$2/$5.bck\"!"
     fi
  else
     echo "  New delivery \"$2/$5\"..."
     newdelivery=1
  fi
  cp "$1/$4" "$3/$5.new" >/dev/null 2>&1
  if [ $? -ne 0 ]
  then
     [ -f "$2/$5" ] && rm -f "$3/$5.old"
     die 1 "Cannot create backup copy of \"$1/$4\" as \"$3/$5.new\"!"
  fi
  cp "$1/$4" "$2/$5" >/dev/null 2>&1
  if [ $? -ne 0 ]
  then
     [ -f "$2/$5" ] && rm -f "$3/$5.old"
     rm -f "$3/$5.new"
     die 1 "Cannot update file \"$2/$5\" with file \"$1/$4\"!"
  fi
  [ "$newdelivery" -eq "1" ] && chmod 770 "$2/$5"
}

######################################################################################
# Name: create_indirs() 
# Arguments:
#   $1 - root dir
#   $2 - inner dir
# Purpose: 
#   Function for create needed internal dirs from the specified path in some root dir. 
######################################################################################
create_indirs() {
  [ ! -d "$1" ] && die 1 "Dirrectory \"$1\" for creating internal directory does not exist!"
  [ -z "$2" ] && return
  [ -d "$1/$2" ] && return
  dir="$1"
  for indir in `echo "$2" | awk -F/ '{for(i=1;i<=NF;i++)print $i;}'`
  do
     dir="$dir/$indir" 
     mkdir "$dir" >/dev/null 2>&1 || die 1 "Cannot create internal directory \"$dir\"!"
  done
} 

###################################################################################### 
# Name: getpath() 
# Arguments: 
#   $1 - file with path 
# Purpose: 
#   Function for getting path from full file name, mean file with path. As since, this 
#   function using for internal dirs, it return path without first slash "/". 
###################################################################################### 
getpath() {
  echo "$1" | awk -F/ '{if(NF>1){printf "%s",$1;for(i=2;i<NF;i++)printf "/%s",$i;}}'
}

###################################################################################### 
# Name: getBinExeStatus() 
# Arguments: 
#   $1 - file with path 
# Purpose: 
#   Function for recognize binary executable files. 
######################################################################################
getBinExeStatus() { 
  file "$1" | grep "exe" >/dev/null 2>&1 
  if [ $? -eq 0 ] 
  then 
     file "$1" | grep "script" >/dev/null 2>&1 
     if [ $? -ne 0 ]
     then 
        echo "1" 
        return 
     fi 
  fi 
  echo "0" 
} 
 
###################################################################################### 
# Name: getAgentID() 
# Arguments: 
#   (none)
# Purpose: 
#   Function for getting process ID of "agent". Process ID needed for checking is the
#   "agent" already started and for killing it by "kill" command, as since "pgrep" and
#   "pkill" does not exist on aix.
###################################################################################### 
getAgentID() {
  if [ "$SYSTEM" = "OS/390" ]
  then
     ps -e | grep "agent" | awk '{if($4=="../bin/agent")print $1;}'
  else  
     ps -e | grep "agent" | awk '{if($4=="agent")print $1;}'
  fi  
}


###################################################################################### 
# Name: do_files_upgrade()
# Arguments: 
#   (none)
# Purpose: 
#   Function for doing lst-files pgrade operation from tar archive.
###################################################################################### 
files_upgrade() {
  #Extract tar-archive:
  echo "Files extracting..."
  [ ! -f "$CURDIR/$FLIST" ] && die 1 "File \"$CURDIR/$FLIST\" does not exist!"
  cd "$TEMPDIR" >/dev/null 2>&1 || die 1 "Cannot change directory to temporary folder \"$TEMPDIR\"!"
  if [ "$SYSTEM" = "OS/390" ]
  then
     tar xvfo "$CURDIR/$FTAR" >/dev/null 2>&1 \
        || die 1 "Cannot extract \"$CURDIR/$FTAR\" archive!"
  else
     tar xvf "$CURDIR/$FTAR" >/dev/null 2>&1 \
        || die 1 "Cannot extract \"$CURDIR/$FTAR\" archive!"
  fi      
  #Update list of files from file $FLIST.
  echo "Files updating..."
  for filewithpath in `cat "$CURDIR/$FLIST" | grep -v "^#" | sort -u` 
  do 
     filename=`basename "$filewithpath"` 
     pckgdir=`getpath "$filewithpath"` 
     if [ -f "$TEMPDIR/$pckgdir/$filename" ]
     then 
        fullarchdir="$TEMPDIR/$pckgdir" 
     else 
        fullarchdir="" 
        for archdir in `find "$TEMPDIR/$SUBMAPPERDIR" -name "$filename"` 
        do 
           newfilename=`basename $archdir` 
           if [ "$newfilename" = "$filename" ] 
           then 
              newarchdir=`getpath "$archdir"` 
              if [ -f "$newarchdir/$filename" ] 
              then 
                 fullarchdir="$newarchdir" 
                 break 
              fi 
           fi 
        done 
        [ -n "$fullarchdir" ] || die 1 "Archive \"$CURDIR/$FTAR\" does not contain file \"$filename\"!" 
     fi 
     create_indirs "$BACKUPDIR" "$pckgdir"
     update_file "$fullarchdir" \
                 "$CEAACTDIR/$pckgdir" \
                 "$BACKUPDIR/$pckgdir" \
                 "$filename" \
                 "$filename"
     #Update each executable files with their .msg file. 
     fileBinExeStatus=`getBinExeStatus "$CEAACTDIR/$pckgdir/$filename"`
     if [ "$fileBinExeStatus" -eq "1" ]
     then
        update_file "$TEMPDIR/$MSGDIR" \
                    "$CEAACTDIR/$pckgdir" \
                    "$BACKUPDIR/$pckgdir" \
                    "$FMSG" \
                    "$filename.msg" 
     fi 
  done
}

######################################################################################
# Start script with argument checking.

if [ -n "$1" ]
then 
  CEAACTDIR="$1"
else
  echo
  echo "Using: $0 <IBM Tivoli Compliance Insight Manager Actuator directory>"
  echo "       Default IBM Tivoli Compliance Insight Manager Actuator directory:"
  echo "         AIX:  \"/usr/lpp/IBM/TCIM/actuator\""
  echo "         HPUX: \"/opt/IBM/TCIM/actuator\""
  echo "         SUN:  \"/opt/IBM/TCIM/actuator\""
  echo
  exit 2
fi

#Check Actuator directory.
[ -d "$CEAACTDIR" ] || die 2 "Actuator directory \"$CEAACTDIR\" does not exist!"
##echo "$CEAACTDIR" | grep -e "^\/" || die 2 "Actuator directory should be specified as an absolute path!"
##As since for sun default grep has limited regular expresion. 
echo "$CEAACTDIR" | awk -F/ '{if($1=="") exit 0; else exit 1;}' \
     || die 2 "Actuator directory should be specified as an absolute path!"
echo
echo "Starting hotfix installation.." 

#If can you restore running of actuator stop hotfix installation.
[ -x "$CEAACTDIR/$CEACLIENT" ] || die 2 "Cannot find IBM Tivoli Compliance Insight Manager Actuator client \"$CEAACTDIR/$CEACLIENT\"!"

#Keep current directory.
CURDIR="`pwd`"

#Create temporary and backup directories.
mkdir "$TEMPDIR" >/dev/null 2>&1 || die 1 "Cannot create temporary directory \"$TEMPDIR\"!"
chmod o= "$TEMPDIR"
mkdir "$BACKUPDIR" >/dev/null 2>&1 || die 1 "Cannot create backup directory \"$BACKUPDIR\"!"
chmod o= "$BACKUPDIR"

#Export variables needed for subscripts running.
export CEAACTDIR CEACLIENT SUBMAPPERDIR MSGDIR FMSG FLIST FTAR TEMPDIR BACKUPDIR SYSTEM

pre_executables_run
stop_watchdog || die 1 "\"Automatically restart\" functionality could not be switched off!"
stop_agent || die 1 "IBM Tivoli Compliance Insight Manager Actuator deamon could not be stoped!"
files_upgrade
in_executables_run
start_agent || die 1 "IBM Tivoli Compliance Insight Manager Actuator deamon could not be restarted!"
start_watchdog || die 1 "\"Automatically restart\" functionality could not be switched on!"
post_executables_run

die 0 "Hotfix installed SUCCESSFULLY!"
