#!/bin/sh
###########################################################BeginFile############
# Copyright:  (c) 2000 by Consul Risk Management B.V.  All Rights Reserved
#
# Time-stamp: <030508 edwin 10:30 genact.sh>
#
# Name:
#   genact.sh
#
# Purpose:
#   Generic actuator script for HP-UX platform. It collects 4 sublogs:
#     1. The audit trail
#     2. The password file
#     3. The group file
#     4. An auditID to username mapping file determined from the auth database 
#
# Notes:
#   The used command interpreter (shell) needs to support the following
#   features:
#     o More than 9 command line parameters.
#     o The [[ expression ]] construct.
#     o The explicit 'override noclobber option' redirection symbol >|.
#     o The |& operator which spawns a subprocess and establishes a 2-way pipe
#       to the parent shell.
#   The default HP-UX /bin/sh supports all this.
#
# History:
#   000616 <Version> arthur <ID>: Created.
#   000620 <Version> arthur <ID>: First revision I'm content about.
###########################################################EndFile##############

# Die function. Removes tempfiles, optionally prints a message to stderr, and exits.
die ()
{
    rm -f audsys.out
    [[ -n "$2" ]] && echo "$0: $2" 1>&2
    exit $1
}

# To disable debugging, do not set CEA_GENACT_DEBUG_LOG.
CEA_GENACT_DEBUG_LOG=../log/debug.log

# Debug function. Reads debug message from stdin when no parameters are given,
# otherwise prints whatever is on the command line to the debug log.
debug ()
{
    if [[ -n "$CEA_GENACT_DEBUG_LOG" ]]
    then
        if [[ $# -ne 0 ]]
        then
            echo "$*" >> $CEA_GENACT_DEBUG_LOG
        else
            while read line
            do
                echo "$line" >> $CEA_GENACT_DEBUG_LOG
            done
        fi
    fi
}


testrights ()
{
  FILENAME=$1
  ACC=$2

  [ -n "$FILENAME" ] || die 999 "Internal Error: Function 'testrights' needs at least 1 argument."
  [ -n "$ACC" ] || return 0    # Do nothing if no rights specified.
  if [ "`echo $ACC | grep R`" ]
  then
    test -r $FILENAME || die 1 "File $FILENAME is not accesible for reading"
  fi
  if [ "`echo $ACC | grep W`" ]
  then
    test -w $FILENAME || die 1 "File $FILENAME is not accesible for writing"
  fi
  if [ "`echo $ACC | grep X`" ]
  then
    test -x $FILENAME || die 1 "File $FILENAME is not accesible for executing"
  fi
}

# Check if we're executed with a correct working directory.
[[ "`basename $PWD`" = "run" && -d ../bin && -d ../log ]] || die 1 "Must be executed from run directory."

# Start new debug log. 
[[ -n "$CEA_GENACT_DEBUG_LOG" ]] && echo "CEA/ExtendIT Debug Log for $0 opened on `date`." >| $CEA_GENACT_DEBUG_LOG

# Get properties as passed by ExtendIT toolkit.
[[ "$1" != " " ]] && TEMPFILE1=$1
[[ "$2" != " " ]] && TEMPFILE2=$2
[[ "$3" != " " ]] && TEMPFILE3=$3
[[ "$4" != " " ]] && TEMPFILE4=$4
[[ "$5" != " " ]] && OPTION1=$5
[[ "$6" != " " ]] && OPTION2=$6
[[ "$7" != " " ]] && OPTION3=$7
[[ "$8" != " " ]] && OPTION4=$8
[[ "$9" != " " ]] && OPTION5=$9
[[ "${10}" != " " ]] && OPTION6=${10}
[[ "${11}" != " " ]] && OPTION7=${11}
[[ "${12}" != " " ]] && OPTION8=${12}

debug "TEMPFILE1 = '$TEMPFILE1'"
debug "TEMPFILE2 = '$TEMPFILE2'"
debug "TEMPFILE3 = '$TEMPFILE3'"
debug "TEMPFILE4 = '$TEMPFILE4'"
debug "OPTION1   = '$OPTION1'"
debug "OPTION2   = '$OPTION2'"
debug "OPTION3   = '$OPTION3'"
debug "OPTION4   = '$OPTION4'"
debug "OPTION5   = '$OPTION5'"
debug "OPTION6   = '$OPTION6'"
debug "OPTION7   = '$OPTION7'"
debug "OPTION8   = '$OPTION8'"

# Check if required arguments are present.
if [[ -z "$TEMPFILE1" || -z "$TEMPFILE2" || -z "$TEMPFILE3" || -z "$TEMPFILE4" ]]
then
    die 2 "Not all required logplatforminstance properties (tempfile 1 to 4) are given."
fi

# Set symbols used in rest of script.
SUBLOG1=$TEMPFILE1
SUBLOG2=$TEMPFILE2
SUBLOG3=$TEMPFILE3
SUBLOG4=$TEMPFILE4

BIN1=${OPTION1:-"/var/aud/adm/bin1"}
BIN2=${OPTION2:-"/var/aud/adm/bin2"}
BINSIZE=${OPTION3:-"1024"}

PASSWD=/etc/passwd
GROUP=/etc/group

debug "SUBLOG1   = '$SUBLOG1'"
debug "SUBLOG2   = '$SUBLOG2'"
debug "SUBLOG3   = '$SUBLOG3'"
debug "SUBLOG4   = '$SUBLOG4'"
debug "BIN1      = '$BIN1'"
debug "BIN2      = '$BIN2'"
debug "BINSIZE   = '$BINSIZE'"

# This should not be necessary, but hey.
rm -f $SUBLOG1
rm -f $SUBLOG2
rm -f $SUBLOG3
rm -f $SUBLOG4

# Check if we are 'root' user
if [ `id -u` -eq 0 ]
then
    SUDO=""
else
    SUDO="sudo"
fi

# Get sublog 1, the audit trail.

# Parse audsys output to find the names of the current and the next
# audit log file.
set `$SUDO audsys | grep " file:"` && CURRENT=$3 && NEXT=$6

debug "CURRENT   = '$CURRENT'"
debug "NEXT      = '$NEXT'"

# Check if parsing was successful.
[[ -n "$CURRENT" && -n "$NEXT" ]] || die 3 "Error parsing audsys output."

# Check if names of bin1 and bin2 are consistent with audsys output.
if [[ ( $CURRENT != $BIN1 && $CURRENT != $BIN2 ) ||
      ( $NEXT != $BIN1 && $NEXT != $BIN2 && $NEXT != "none" ) ]]
then
    die 4 "Log file names passed as options 1 and 2 are inconsistent with audsys output."
fi

# Check if the audit system has already switched bins automatically. If so,
# get the log that has filled up, and point the NEXT var to it.
if [[ $NEXT = "none" ]]
then
    if [[ $CURRENT = $BIN1 ]]
    then
        NEXT=$BIN2
    else
        NEXT=$BIN1
    fi

    debug "NEXT      = '$NEXT'"

    if $SUDO test -s $NEXT
    then
        $SUDO cat $NEXT >> $SUBLOG1
        $SUDO rm $NEXT
    fi
fi

# Check if auditing is currently on (LS QE110003)
AUDITSTATUS=`$SUDO audsys | grep "currently on"`
if test -z "$AUDITSTATUS" ; then
  AUDITSTATUS=off
else
  AUDITSTATUS=on
fi

# Switch to the next bin; if auditing is currently on (LS QE110003)
if test $AUDITSTATUS = on ; then
  if ! $SUDO audsys -c $NEXT -s $BINSIZE > /dev/null 2>| audsys.out
  then
      cat audsys.out 1>&2
      die 5   # No message; the audsys output serves as one.
  fi
  # 20050914/IgorSo/QE0507081
  sleep 5
fi

# Save the current bin.
if $SUDO test -s $CURRENT
then
    $SUDO cat $CURRENT >> $SUBLOG1
    $SUDO rm $CURRENT
fi

# Ensure existence of sublog 1
touch $SUBLOG1

# Set next bin; if auditing is currently on (LS QE110003)
if test $AUDITSTATUS = on ; then
  $SUDO audsys -x $CURRENT -z $BINSIZE >/dev/null || die 6   # No message; audsys will print one.
fi

$SUDO audsys | debug

# Get sublog 2, the passwd file.
if $SUDO test -f $PASSWD
then
    cp $PASSWD $SUBLOG2
    chmod u+w $SUBLOG2
else
    touch $SUBLOG2
fi

# Get sublog 3, the group file.
if $SUDO test -f $GROUP
then
    cp $GROUP $SUBLOG3
    chmod u+w $SUBLOG3
else
    touch $SUBLOG3
fi

# Get sublog 4, the auth db.
testrights ../bin/getaid "X"
[[ -x ../bin/getaid ]]   || die 7 "Can not execute ../bin/getaid."
../bin/getaid > $SUBLOG4 || die 8 "Unknown error executing ../bin/getaid."

# Clean up and exit successfully.
die 0
